use crate::model::playlist::Playlist;
use crate::model::scene::Scene;
use log::info;
use poise::serenity_prelude::ScheduledEventId;
use serde::{Deserialize, Serialize};
use std::path::PathBuf;

#[derive(Debug, Clone, Default, Serialize, Deserialize)]
pub struct Session {
    pub name: String,
    pub event: ScheduledEventId,
    pub scenes: Vec<Scene>,
}

#[derive(Debug, Clone, Default, Serialize, Deserialize)]
pub struct SessionConfig {
    pub active_session: Option<ScheduledEventId>,
    pub playlists: Vec<Playlist>,
    pub global_scenes: Vec<Scene>,
    pub sessions: Vec<Session>,
}

impl SessionConfig {
    pub async fn init(path: PathBuf) -> Result<Self, std::io::Error> {
        if path.exists() {
            info!("Initializing with config at {:?}...", path);
            Self::load_config(path).await
        } else {
            info!("Initializing with a blank config...");
            let config = Self::default();

            config.save_config(path).await?;
            Ok(config)
        }
    }

    pub async fn load_config(path: PathBuf) -> Result<Self, std::io::Error> {
        let file = tokio::fs::read(path).await?;

        let toml_string = String::from_utf8(file).unwrap();

        Ok(toml::from_str(&toml_string).unwrap())
    }

    pub async fn save_config(&self, path: PathBuf) -> Result<(), std::io::Error> {
        tokio::fs::write(path, toml::to_string(self).unwrap()).await
    }

    pub fn create_session(&mut self, event: ScheduledEventId, name: &str) {
        self.sessions.push(Session {
            name: name.to_string(),
            event,
            scenes: vec![],
        });
    }

    pub fn remove_session(&mut self, event: ScheduledEventId) {
        self.sessions.retain(|s| s.event != event);
    }
}
